package cn.orm.util.cache;

import cn.component.RedisKit;
import cn.component.context.SpringContextKit;
import cn.constant.CommonConst;
import cn.pojo.entity.TSysDictData;
import cn.util.core.IsKit;
import cn.util.core.TypeKit;
import java.util.Collection;
import java.util.List;
import org.apache.commons.lang3.StringUtils;

/**
 * 字典工具类
 */
public class DictCacheUtil
{
    /**分隔符*/
    public static final String SEPARATOR = ",";

    /**
     * 设置字典缓存
     *
     * @param key 参数键
     * @param dictDatas 字典数据列表
     */
    public static void setDictCache(String key, List<TSysDictData> dictDatas)
    {
    	SpringContextKit.getBean(RedisKit.class).setCacheObject(getCacheKey(key), dictDatas);
    }

    /**
     * 获取字典缓存
     *
     * @param key 参数键
     * @return dictDatas 字典数据列表
     */
    public static List<TSysDictData> getDictCache(String key)
    {
    	Object cacheObj = SpringContextKit.getBean(RedisKit.class).getCacheObject(getCacheKey(key));
        if (IsKit.isNotNull(cacheObj)){
            List<TSysDictData> dictDatas = TypeKit.cast(cacheObj);
            return dictDatas;
        }
        return null;
    }

    /**
     * 根据字典类型和字典值获取字典标签
     *
     * @param dictType 字典类型
     * @param dictValue 字典值
     * @return 字典标签
     */
    public static String getDictLabel(String dictType, String dictValue)
    {
        return getDictLabel(dictType, dictValue, SEPARATOR);
    }

    /**
     * 根据字典类型和字典标签获取字典值
     *
     * @param dictType 字典类型
     * @param dictLabel 字典标签
     * @return 字典值
     */
    public static String getDictValue(String dictType, String dictLabel)
    {
        return getDictValue(dictType, dictLabel, SEPARATOR);
    }

    /**
     * 根据字典类型和字典值获取字典标签
     *
     * @param dictType 字典类型
     * @param dictValue 字典值
     * @param separator 分隔符
     * @return 字典标签
     */
    public static String getDictLabel(String dictType, String dictValue, String separator)
    {
        StringBuilder propertyString = new StringBuilder();
        List<TSysDictData> datas = getDictCache(dictType);

        if (StringUtils.containsAny(separator, dictValue) && IsKit.isNotEmpty(datas)){
            for (TSysDictData dict : datas){
                for (String value : dictValue.split(separator)){
                    if (value.equals(dict.getDictValue())){
                        propertyString.append(dict.getDictLabel() + separator);
                        break;
                    }
                }
            }
        }
        else {
            for (TSysDictData dict : datas) {
                if (dictValue.equals(dict.getDictValue())){
                    return TypeKit.objToStr(dict.getDictLabel());
                }
            }
        }
        return StringUtils.stripEnd(propertyString.toString(), separator);
    }

    /**
     * 根据字典类型和字典标签获取字典值
     *
     * @param dictType 字典类型
     * @param dictLabel 字典标签
     * @param separator 分隔符
     * @return 字典值
     */
    public static String getDictValue(String dictType, String dictLabel, String separator)
    {
        StringBuilder propertyString = new StringBuilder();
        List<TSysDictData> datas = getDictCache(dictType);

        if (StringUtils.containsAny(separator, dictLabel) && IsKit.isNotEmpty(datas)){
            for (TSysDictData dict : datas){
                for (String label : dictLabel.split(separator)){
                    if (label.equals(dict.getDictLabel())){
                        propertyString.append(dict.getDictValue() + separator);
                        break;
                    }
                }
            }
        }
        else{
            for (TSysDictData dict : datas){
                if (dictLabel.equals(dict.getDictLabel())){
                    return TypeKit.objToStr(dict.getDictValue());
                }
            }
        }
        return StringUtils.stripEnd(propertyString.toString(), separator);
    }

    /**
     * 清空字典缓存
     */
    public static void clearDictCache()
    {
        Collection<String> keys = SpringContextKit.getBean(RedisKit.class).keys(CommonConst.SYS_DICT_KEY + "*");
        SpringContextKit.getBean(RedisKit.class).deleteObject(keys);
    }

    /**
     * 设置cache key
     *
     * @param configKey 参数键
     * @return 缓存键key
     */
    public static String getCacheKey(String configKey)
    {
        return CommonConst.SYS_DICT_KEY + configKey;
    }

    /**
     * 清空字典缓存
     */
    public static void clearDictCache(String dictType )
    {
        Collection<String> keys = SpringContextKit.getBean(RedisKit.class).keys(CommonConst.SYS_DICT_KEY + dictType);
        SpringContextKit.getBean(RedisKit.class).deleteObject(keys);
    }
}

