package cn.report.template.sam;

import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.poi.xwpf.usermodel.XWPFRun;
import org.apache.poi.xwpf.usermodel.XWPFTable;
import org.apache.poi.xwpf.usermodel.XWPFTableCell;
import org.apache.poi.xwpf.usermodel.XWPFTableRow;
import org.apache.poi.xwpf.usermodel.XWPFParagraph;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlObject;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTRow;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTTcPr;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.CTVMerge;
import org.openxmlformats.schemas.wordprocessingml.x2006.main.STMerge;
import cn.report.template.poi.XWPFTemplate;
import cn.report.template.poi.data.TableRenderData;
import cn.report.template.poi.data.Tables;
import cn.report.template.poi.data.style.BorderStyle;
import cn.report.template.poi.exception.RenderException;
import cn.report.template.poi.policy.RenderPolicy;
import cn.report.template.poi.render.compute.EnvModel;
import cn.report.template.poi.render.compute.RenderDataCompute;
import cn.report.template.poi.render.processor.DocumentProcessor;
import cn.report.template.poi.render.processor.EnvIterator;
import cn.report.template.poi.resolver.TemplateResolver;
import cn.report.template.poi.template.ElementTemplate;
import cn.report.template.poi.template.MetaTemplate;
import cn.report.template.poi.template.run.RunTemplate;
import cn.report.template.poi.util.ReflectionUtils;
import cn.report.template.poi.util.TableTools;


public class CustomTableRenderPolicy implements RenderPolicy 
{
    private String prefix;
    private String suffix;
    private boolean onSameLine;
 
    public CustomTableRenderPolicy() {
        this(false);
    }
 
    public CustomTableRenderPolicy(boolean onSameLine) {
        this("[", "]", onSameLine);
    }
 
    public CustomTableRenderPolicy(String prefix, String suffix) {
        this(prefix, suffix, false);
    }
 
    public CustomTableRenderPolicy(String prefix, String suffix, boolean onSameLine) {
        this.prefix = prefix;
        this.suffix = suffix;
        this.onSameLine = onSameLine;
    }
 
    @Override
    public void render(ElementTemplate eleTemplate, Object data_array, XWPFTemplate template) 
    {
        Object data = null;
        Map<?, ?> data_array_map= (Map<?, ?>) data_array;
        data = data_array_map.get("data_list");
        List<Map> row_map_list = (List<Map>) data_array_map.get("row_map_list");
        RunTemplate runTemplate = (RunTemplate) eleTemplate;
        XWPFRun run = runTemplate.getRun();
        try {
            if (!TableTools.isInsideTable(run)) {
                throw new IllegalStateException(
                        "The template tag " + runTemplate.getSource() + " must be inside a table");
            }
            XWPFTableCell tagCell = (XWPFTableCell) ((XWPFParagraph) run.getParent()).getBody();
            XWPFTable table = tagCell.getTableRow().getTable();
            run.setText("", 0);
 
                
            int templateRowIndex = getTemplateRowIndex(tagCell);
            if (null != data && data instanceof Iterable) 
            {
                Iterator<?> iterator = ((Iterable<?>) data).iterator();
                XWPFTableRow templateRow = table.getRow(templateRowIndex);
                int insertPosition = templateRowIndex;
 
                TemplateResolver resolver = new TemplateResolver(template.getConfig().copy(prefix, suffix));
                boolean firstFlag = true;
                int index = 0;
                boolean hasNext = iterator.hasNext();
                // 循环插入行
                while (hasNext) {
                    Object root = iterator.next();
                    hasNext = iterator.hasNext();
 
                    insertPosition = templateRowIndex++;

                    XWPFTableRow nextRow = table.insertNewTableRow(insertPosition);
                    setTableRow(table, templateRow, insertPosition);
 
                    // double set row
                    XmlCursor newCursor = templateRow.getCtRow().newCursor();
                    newCursor.toPrevSibling();
                    XmlObject object = newCursor.getObject();
                    nextRow = new XWPFTableRow((CTRow) object, table);
                    if (!firstFlag) {
                        // update VMerge cells for non-first row
                        List<XWPFTableCell> tableCells = nextRow.getTableCells();
                        for (XWPFTableCell cell : tableCells) {
                            CTTcPr tcPr = TableTools.getTcPr(cell);
                            CTVMerge vMerge = tcPr.getVMerge();
                            if (null == vMerge) continue;
                            if (STMerge.RESTART == vMerge.getVal()) {
                                vMerge.setVal(STMerge.CONTINUE);
                            }
                        }
                    } else {
                        firstFlag = false;
                    }
                    setTableRow(table, nextRow, insertPosition);
 
                    RenderDataCompute dataCompute = template.getConfig()
                            .getRenderDataComputeFactory()
                            .newCompute(EnvModel.of(root, EnvIterator.makeEnv(index++, hasNext)));
                    List<XWPFTableCell> cells = nextRow.getTableCells();
                    cells.forEach(cell -> {
                        List<MetaTemplate> templates = resolver.resolveBodyElements(cell.getBodyElements());
                        new DocumentProcessor(template, resolver, dataCompute).process(templates);
                    });
                }
                //合并同列的若干行0标识第0列
                 for(Map row_map:row_map_list){
                	 //处理第一列机组合并merge several rows of cells in the same column
                     TableTools.mergeCellsVertically(table, 0, (int)row_map.get("fromrow"), (int)row_map.get("torow"));
                 }
                 /**合并同行某些列 第二行的第二列到第三列(从0开始)【2, 1,2】
                 for(Map row_map:row_map_list){
                	 //merge several columns of cells in the same row, but do'not remove extra cells
                	 TableTools.mergeCellsHorizonal(table,2,1,2);
                     //TableTools.mergeCellsHorizonal(table, 1, (int)row_map.get("fromrow"), (int)row_map.get("torow"));
                 }
                 */
                 
            }
 
            table.removeRow(templateRowIndex);
            afterloop(table, data);
            
        } catch (Exception e) {
            throw new RenderException("HackLoopTable for " + eleTemplate + "error: " + e.getMessage(), e);
        }
    }
 
    private int getTemplateRowIndex(XWPFTableCell tagCell) {
        XWPFTableRow tagRow = tagCell.getTableRow();
        return onSameLine ? getRowIndex(tagRow) : (getRowIndex(tagRow) + 1);
    }
 
    protected void afterloop(XWPFTable table, Object data) {
    }
 
    @SuppressWarnings("unchecked")
    private void setTableRow(XWPFTable table, XWPFTableRow templateRow, int pos) {
        List<XWPFTableRow> rows = (List<XWPFTableRow>) ReflectionUtils.getValue("tableRows", table);
        rows.set(pos, templateRow);
        table.getCTTbl().setTrArray(pos, templateRow.getCtRow());
    }
 
    private int getRowIndex(XWPFTableRow row) {
        List<XWPFTableRow> rows = row.getTable().getRows();
        return rows.indexOf(row);
    }
}
