-- N1 字典总表：系统字典类型(字典的具体值见t_sys_dict_data) ATV::sys_dict
DROP TABLE IF EXISTS `t_sys_dict_type`;
CREATE TABLE `t_sys_dict_type`  (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '字典主键',
  `name` varchar(80) COMMENT '字典名称',
  `type` varchar(80) COMMENT '字典类型',
  `status` char(1) DEFAULT '0' COMMENT '状态（0正常 1停用）',
  `create_by` varchar(64) COMMENT '创建者',
  `create_time` datetime(0) COMMENT '创建时间',
  `update_by` varchar(64) COMMENT '更新者',
  `update_time` datetime(0) COMMENT '更新时间',
  `remark` varchar(100) COMMENT '备注',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE INDEX `dict_type`(`type`) USING BTREE
)ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '字典类型表';

-- N2 字典数据  ATV::sys_dict_item
DROP TABLE IF EXISTS `t_sys_dict_data`;
CREATE TABLE `t_sys_dict_data`  (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '字典编码',
  `dict_sort` int(4) NULL DEFAULT 0 COMMENT '字典排序',
  `dict_label` varchar(50) COMMENT '字典标签(数据项名称/字典项文本)',
  `dict_value` varchar(100) COMMENT '字典键值(数据项值)',
  `dict_type` varchar(80) COMMENT '字典类型', -- t_sys_dict_type.type
  `status` char(1) DEFAULT '0' COMMENT '状态（0正常 1停用）',
  `create_by` varchar(64) COMMENT '创建者',
  `create_time` datetime(0) COMMENT '创建时间',
  `update_by` varchar(64) COMMENT '更新者',
  `update_time` datetime(0) COMMENT '更新时间',
  `remark` varchar(50) COMMENT '备注',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '字典数据表';

	
-- N3 日志管理：用户登录日志
DROP TABLE IF EXISTS `t_sys_log_login`;
CREATE TABLE `t_sys_log_login`  (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '访问ID',
  `user_name` varchar(30) COMMENT '用户账号',
  `ipaddr` varchar(30) COMMENT '登录IP地址',
  `login_location` varchar(100) COMMENT '登录地点',
  `browser` varchar(30) COMMENT '浏览器类型',
  `os` varchar(30) COMMENT '操作系统',
  `status` char(1) COMMENT '登录状态', -- 0成功 1失败
  `msg` varchar(300) COMMENT '提示消息',
  `login_time` datetime(0) COMMENT '访问时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '系统访问记录';
		
-- N4 日志管理：操作日志(用户)
DROP TABLE IF EXISTS `t_sys_log_op`;
CREATE TABLE `t_sys_log_op` (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '日志主键',
  `title` varchar(50) COMMENT '模块标题',
  `business_type` int(2) DEFAULT 0 COMMENT '操作类型', -- log.businessType().ordinal()
  `method` varchar(80) COMMENT '方法名称',
  `request_method` varchar(10) COMMENT '请求方式',
  `operator_type` int(1) DEFAULT 0 COMMENT '操作类别', -- 0其它 1后台用户 2手机端用户
  `oper_name` varchar(50) COMMENT '操作人员',
  `dept_name` varchar(50) COMMENT '部门名称',
  `oper_url` varchar(300) COMMENT '请求URL',
  `oper_ip` varchar(50) COMMENT '主机地址',
  `oper_location` varchar(80) COMMENT '操作地点',
  `oper_param` varchar(300) COMMENT '请求参数',
   --  [mysqld]sql-mode="STRICT_TRANS_TABLES" mediumtext 超长自动截断
  `json_result` varchar(300) COMMENT '返回参数',
  `status` int(1) DEFAULT 0 COMMENT '操作状态', -- 0正常 1异常
  `error_msg` varchar(200) COMMENT '错误消息',
  `oper_time` datetime(0) COMMENT '操作时间',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '操作日志记录';

-- N5 定时任务日志列表
-- 状态异常1 列表中 详细一项显示内容 否则不显示内容 
DROP TABLE IF EXISTS `t_sys_job_log`;
CREATE TABLE `t_sys_job_log`  (
  `id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '任务日志ID',
  `job_name` varchar(64) COMMENT '任务名称',
  `job_group` varchar(64) COMMENT '任务组名',
  `invoke_target` varchar(500) COMMENT '调用目标字符串',
  `job_message` varchar(500) COMMENT '日志信息',
  `status` char(1) COMMENT '执行状态', -- 0正常 1失败
  `exception_info` varchar(400) DEFAULT '' COMMENT '异常信息',
  `create_time` datetime(0) COMMENT '创建时间',
  PRIMARY KEY (`id`) USING BTREE
)ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '定时任务调度日志表';
INSERT INTO `t_sys_job_log` VALUES (4, '测试', 'DEFAULT', 'moodingTask.mdNoParams', '0/15 * * * * ?', '1',  'mooding', '2021-12-29 22:29:55');
INSERT INTO `t_sys_job_log` VALUES (5, '有参数', 'SYSTEM', 'cn.mooding.modules.quartz.task.MoodingTask.mdParams(\'木丁科技\')', '0/10 * * * * ?', '1', 'admin', '2021-12-29 22:29:59');


-- N6 定时任务调度表 每隔5秒执行一次：*/5 * * * * ?
DROP TABLE IF EXISTS `t_sys_job`;
CREATE TABLE `t_sys_job`  (
  `job_id` bigint(20) NOT NULL AUTO_INCREMENT COMMENT '任务ID',
  `job_name` varchar(64) COMMENT '任务名称',
  `job_group` varchar(64) DEFAULT 'DEFAULT' COMMENT '任务组名',
  `invoke_target` varchar(500) COMMENT '调用目标字符串',
  `cron_expression` varchar(255) COMMENT 'cron执行表达式',
  `misfire_policy` varchar(20) DEFAULT '3' COMMENT '计划执行错误策略', -- 1立即执行 2执行一次 3放弃执行
  `concurrent` char(1) DEFAULT '1' COMMENT '是否并发执行', -- 0允许 1禁止
  `status` char(1) COMMENT '状态（0正常 1暂停）',
  `create_by` varchar(64) DEFAULT '' COMMENT '创建者',
  `create_time` datetime(0) COMMENT '创建时间',
  `update_by` varchar(64) DEFAULT '' COMMENT '更新者',
  `update_time` datetime(0) COMMENT '更新时间',
  `remark` varchar(200) DEFAULT '' COMMENT '备注信息',
   PRIMARY KEY (`job_id`) USING BTREE,
   UNIQUE KEY `UNIQUE` (`job_name`, `job_group`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '定时任务调度表';
INSERT INTO `t_sys_job` VALUES (1, '测试', 'DEFAULT', 'Task.mdNoParams', '*/5 * * * * ?', '1', '0', '1', 'admin', '2021-07-26 22:54:04', 'mooding', '2021-12-29 22:29:55', '');
INSERT INTO `t_sys_job` VALUES (2, '有参数', 'SYSTEM', 'cn.service.task.Task.mdParams(\'实验技\')', '0/10 * * * * ?', '1', '0', '1', 'admin', '2021-07-27 07:15:13', 'mooding', '2021-12-29 22:29:59', '');


-- N7 岗位或职务信息 ATV:sys_position：dept_id=sys_org_code所属组织机构编码
DROP TABLE IF EXISTS `t_sys_post`;
CREATE TABLE `t_sys_post`  (
  `post_id` varchar(30) COMMENT '岗位编码',
  `post_name` varchar(50) COMMENT '岗位名称',
  `post_sort` int(4) COMMENT '显示顺序',
  `dept_id` varchar(50) COMMENT '所属部门(保留)', -- t_sys_dept.dept_id
  `post_rank` int(4) COMMENT '职级(保留)',
  `company_id` varchar(50) COMMENT '所属公司id(保留)',
  `del_flag` char(1) DEFAULT '0' COMMENT '删除标志', -- 0代表存在 1代表删除
  `status` char(1) COMMENT '状态', -- 0正常 1停用
  `create_by` varchar(50) DEFAULT '' COMMENT '创建者',
  `create_time` datetime(0) COMMENT '创建时间',
  `update_by` varchar(54) DEFAULT '' COMMENT '更新者',
  `update_time` datetime(0) COMMENT '更新时间',
  `remark` varchar(100) COMMENT '备注',
  PRIMARY KEY (`post_id`) USING BTREE,
  UNIQUE KEY `UNIQUE` (`post_name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '岗位信息表';
INSERT INTO `t_sys_post` VALUES (1, '1', 0, '','0','','0', '0', 'admin', '2021-12-13 21:11:32', '', NULL, '');
INSERT INTO `t_sys_post` VALUES (2, '2', 0,'','0','', '0', '0', 'admin', '2021-12-13 21:11:37', '', NULL, '');
INSERT INTO `t_sys_post` VALUES (3, '3', 0, '','0','','0', '0', 'admin', '2021-12-13 21:11:41', '', NULL, '');

-- N8 部门或组织机构信息表 ATV:sys_depart
DROP TABLE IF EXISTS `t_sys_dept`; 
CREATE TABLE `t_sys_dept`  (
  `dept_id` int(20) NOT NULL AUTO_INCREMENT COMMENT '部门id(节点id)',
  `parent_id` int(20) DEFAULT 0 COMMENT '父节点id(上级部门)', -- 0默认根节点
  `ancestors` varchar(50) DEFAULT '' COMMENT '祖级列表id路径(直到根节点)', -- 逗号分割
  `dept_name` varchar(50) COMMENT '部门名称',
  `order_num` int(4) DEFAULT 0 COMMENT '显示顺序',
  `dept_name_en` varchar(50) COMMENT '英文名',
  `dept_name_abbr` varchar(50) COMMENT '缩写',
  `fax` varchar(32) COMMENT '传真',
  `address` varchar(100) COMMENT '地址',
  `leader` varchar(20) COMMENT '负责人',
  `phone` varchar(11) COMMENT '联系电话',
  `email` varchar(50) COMMENT '邮箱',
  `status` char(1) COMMENT '部门状态', -- 0正常 1停用
  `del_flag` char(1) DEFAULT '0' COMMENT '删除标志', -- 0代表存在 1代表删除
  `create_by` varchar(64) COMMENT '创建者',
  `create_time` datetime(0) COMMENT '创建时间',
  `update_by` varchar(64) COMMENT '更新者',
  `update_time` datetime(0) COMMENT'更新时间',
   `remark` varchar(100) COMMENT '备注',
  PRIMARY KEY (`dept_id`) USING BTREE,
  KEY `index_depart_parent_id` (`parent_id`) USING BTREE,
  KEY `index_depart_depart_order` (`order_num`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '部门表';
INSERT INTO `t_sys_dept` VALUES (100, 0, '0', '奥特维科技', 0,'','','','', '王五', '15888888888', 'moodingtech@gmail.com', '0', '0', 'admin', '2018-03-16 11:33:00', 'admin', '2021-11-30 15:38:08','');
INSERT INTO `t_sys_dept` VALUES (101, 100, '0,100', '深圳总公司', 1, '','','','','王五', '15888888888', 'moodingtech@gmail.com', '0', '0', 'admin', '2018-03-16 11:33:00', 'admin', '2021-11-30 15:38:09','');
INSERT INTO `t_sys_dept` VALUES (102, 100, '0,100', '长沙分公司', 2, '','','','','王五', '15888888888', 'moodingtech@gmail.com', '0', '0', 'admin', '2018-03-16 11:33:00', 'admin', '2021-07-15 07:19:36','');
INSERT INTO `t_sys_dept` VALUES (103, 101, '0,100,101', '研发部门', 1,'','','','', '王五', '15888888888', 'moodingtech@gmail.com', '0', '0', 'admin', '2018-03-16 11:33:00', 'admin', '2018-03-16 11:33:00','');
INSERT INTO `t_sys_dept` VALUES (104, 101, '0,100,101', '市场部门', 2,'','','','', '王五', '15888888888', 'moodingtech@gmail.com', '0', '0', 'admin', '2018-03-16 11:33:00', 'admin', '2018-03-16 11:33:00','');
INSERT INTO `t_sys_dept` VALUES (105, 101, '0,100,101', '测试部门', 3,'','','','', '王五', '15888888888', 'moodingtech@gmail.com', '0', '0', 'admin', '2018-03-16 11:33:00', 'admin', '2018-03-16 11:33:00','');
INSERT INTO `t_sys_dept` VALUES (106, 101, '0,100,101', '财务部门', 4,'','','','', '王五', '15888888888', 'moodingtech@gmail.com', '0', '0', 'admin', '2018-03-16 11:33:00', 'admin', '2018-03-16 11:33:00','');
INSERT INTO `t_sys_dept` VALUES (107, 101, '0,100,101', '运维部门', 5,'','','','', '王五', '15888888888', 'moodingtech@gmail.com', '0', '0', 'admin', '2018-03-16 11:33:00', 'admin', '2018-03-16 11:33:00','');
INSERT INTO `t_sys_dept` VALUES (108, 102, '0,100,102', '市场部门', 1,'','','','', '王五', '15888888888', 'moodingtech@gmail.com', '0', '0', 'admin', '2018-03-16 11:33:00', 'admin', '2018-03-16 11:33:00','');
INSERT INTO `t_sys_dept` VALUES (109, 102, '0,100,102', '财务部门', 2, '','','','','张三', '15888888888', 'moodingtech@gmail.com', '0', '0', 'admin', '2018-03-16 11:33:00', 'admin', '2021-07-15 06:42:25','');
INSERT INTO `t_sys_dept` VALUES (111, 102, '0,100,102', '研发部', 3, '','','','','李四', '15888888888', '', '1', '0', 'admin', '2021-07-15 06:55:25', 'admin', '2021-07-15 20:47:47','');
INSERT INTO `t_sys_dept` VALUES (112, 102, '0,100,102', '测试部门', 2,'','','','', '张三', '15588888888', '', '1', '2', 'admin', '2021-08-19 07:12:39', '', NULL,'');

-- N9 参数配置 用途：可以保存SQL语句config_value
DROP TABLE IF EXISTS `t_sys_config`;
CREATE TABLE `t_sys_config`  (
  `id` int(5) NOT NULL AUTO_INCREMENT COMMENT '参数主键',
  `config_name` varchar(80) DEFAULT '' COMMENT '参数名称',
  `config_key` varchar(80) DEFAULT '' COMMENT '参数键名',
  `config_value` varchar(200) DEFAULT '' COMMENT '参数键值sql',
  `config_type` char(1) DEFAULT 'N' COMMENT '系统内置(sys_yes_no)', -- Y是 N否
  `create_by` varchar(64) DEFAULT '' COMMENT '创建者',
  `create_time` datetime(0) COMMENT '创建时间',
  `update_by` varchar(64) DEFAULT '' COMMENT '更新者',
  `update_time` datetime(0) COMMENT '更新时间',
  `remark` varchar(100) COMMENT '备注',
  PRIMARY KEY (`id`) USING BTREE
)ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '参数配置表';

-- N10 编码规则:保存库表的一些字段比如sn的编码规则 ATV::sys_fill_rule
DROP TABLE IF EXISTS `t_sys_fill_rule`;
CREATE TABLE `t_sys_fill_rule` (
  `id` varchar(32) NOT NULL COMMENT '主键ID',
  `rule_name` varchar(80) COMMENT '规则名称',
  `rule_code` varchar(80) COMMENT '规则编码',
  `rule_class` varchar(100) COMMENT '规则实现类',
  `rule_params` varchar(100) COMMENT '规则参数', -- 规则JSON
  `update_by` varchar(32) COMMENT '修改人',
  `update_time` datetime COMMENT '修改时间',
  `create_by` varchar(32) COMMENT '创建人',
  `create_time` datetime COMMENT '创建时间',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE KEY `uni_sys_fill_rule_code` (`rule_code`) USING BTREE
)ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;
INSERT INTO `t_sys_fill_rule` (`id`, `rule_name`, `rule_code`, `rule_class`, `rule_params`, `update_by`, `update_time`, `create_by`, `create_time`) 
VALUES ('1202551334738382850', '机构编码生成', 'org_num_role', 'cn.facade.rule.OrgCodeRule', '{\"parentId\":\"c6d7cb4deeac411cb3384b1b31278596\"}', 'admin', '2019-12-09 10:37:06', 'admin', '2019-12-05 19:32:35');
INSERT INTO `t_sys_fill_rule` (`id`, `rule_name`, `rule_code`, `rule_class`, `rule_params`, `update_by`, `update_time`, `create_by`, `create_time`) 
VALUES ('1202787623203065858', '分类字典编码生成', 'category_code_rule', 'cn.facade.rule.CategoryCodeRule', '{\"pid\":\"\"}', 'admin', '2019-12-09 10:36:54', 'admin', '2019-12-06 11:11:31');
INSERT INTO `t_sys_fill_rule` (`id`, `rule_name`, `rule_code`, `rule_class`, `rule_params`, `update_by`, `update_time`, `create_by`, `create_time`) 
VALUES ('1322136526816100353', '故障单编号', 'fault_sn_rule', 'cn.facade.rule.AtvFaultRule', 'p:FS}', NULL, NULL, 'chenyu', '2020-10-30 19:21:27');
INSERT INTO `t_sys_fill_rule` (`id`, `rule_name`, `rule_code`, `rule_class`, `rule_params`, `update_by`, `update_time`, `create_by`, `create_time`) 
VALUES ('100', '订单编号', 'yh_dingdan', '', 'p:HT,d:yyyyMMdd,r:5,sn:5', NULL, NULL, 'admin', '2023-01-30 19:21:27');


-- N11 动态路由。meta_字段都属于meta域内{}的字 ATV::sys_permission
-- menu_id约定：一级菜单两位数以内；二级菜单三位以内：依次类推
DROP TABLE IF EXISTS `t_sys_menu`;
CREATE TABLE `t_sys_menu`  (
   `menu_id` int(6) NOT NULL COMMENT '菜单ID',
   `meta_title` varchar(50) NOT NULL COMMENT '菜单名称',
   `parent_id` int(20) DEFAULT 0 COMMENT '父菜单ID',
   `menu_sort` int(4) DEFAULT 0 COMMENT '显示顺序',
   `path` varchar(100) DEFAULT '' COMMENT '路径',  -- 路由路径 顶级菜单路径 用于跳转
   `component` varchar(50) COMMENT '组件', -- 组件路径 顶级路由出口.格式：system/user/index

	-- 新加字段
   `name` varchar(100) DEFAULT '' COMMENT '组件名字',  -- 路由名称
   `redirect` varchar(100) COMMENT '重定向 顶级菜单重定向路径',
   `btnPower` varchar(200) COMMENT '权限标识(按钮)',
    -- 1、isLink:true 2、meta_is_link链接地址不为空` 
   `is_link` int(1) COMMENT '是否外链(1 isLink:true 2 链接地址不为空)',
	
     -- 是否内嵌，开启条件，`1、isFrame:true 2、meta_is_link链接地址不为空
	`meta_is_frame` int(1) DEFAULT 1 COMMENT '是否内嵌（0是 1否）',
	`meta_is_link` varchar(100) COMMENT '链接地址', -- 1 isLink:true 2 链接地址不为空
	`menu_type` char(1) COMMENT '菜单类型', -- M目录 C菜单 F按钮
	`meta_is_hide` int(1) DEFAULT 0 COMMENT '显示状态', -- 0显示 1隐藏
	`meta_is_keep_alive` int(1) DEFAULT 0 COMMENT '是否缓存组件状态', -- 0是 1否
	`meta_is_affix` int(1) DEFAULT 0 COMMENT '是否固定在 tagsView 栏上', -- 0是 1否
    `meta_roles` varchar(200) COMMENT '权限标识(页面多选)',
    `meta_icon` varchar(50) DEFAULT '#' COMMENT '菜单图标',
  
   `status` int(1) DEFAULT 0 COMMENT '菜单状态', -- 0正常 1停用
   `create_by` varchar(64) DEFAULT '' COMMENT '创建者',
   `create_time` datetime(0) COMMENT '创建时间',
   `update_by` varchar(64) DEFAULT '' COMMENT '更新者',
   `update_time` datetime(0) COMMENT '更新时间',
   `remark` varchar(100) DEFAULT '' COMMENT '备注',
  PRIMARY KEY (`menu_id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '菜单权限表';
 
-- N12 角色信息表  ATV:sys_role
-- 目前：role_name和role_key可选的内容配置文件中
DROP TABLE IF EXISTS `t_sys_role`;
CREATE TABLE `t_sys_role`  (
  `role_id` int(20) NOT NULL AUTO_INCREMENT COMMENT '角色UUID',
  `role_name` varchar(30) COMMENT '角色名称',
  `role_key` varchar(30) COMMENT '权限字符串[页面菜单]',
  `role_sort` int(4) COMMENT '显示顺序sort',
  -- 1全部数据权限 2自定数据权限 3本部门数据权限 4本部门及以下数据权限
  `data_scope` char(1) DEFAULT '1' COMMENT '数据范围',
  `status` char(1) COMMENT '角色状态', -- 0正常 1停用
  `del_flag` char(1) DEFAULT '0' COMMENT '删除标志', -- 0代表存在 1代表删除
  `create_by` varchar(64) DEFAULT '' COMMENT '创建者',
  `create_time` datetime(0) COMMENT '创建时间',
  `update_by` varchar(64) DEFAULT '' COMMENT '更新者',
  `update_time` datetime(0) COMMENT '更新时间',
  `remark` varchar(100) COMMENT '备注',
  PRIMARY KEY (`role_id`) USING BTREE,
  UNIQUE INDEX `name`(`role_name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '角色信息表';
INSERT INTO `t_sys_role` VALUES (1, '管理员', 'admin', 1, '3', '0', '0', 'admin', '2018-03-16 11:33:00', 'admin', '2021-12-02 16:40:54', '管理员');
INSERT INTO `t_sys_role` VALUES (2, '普通角色', 'common', 2, '2', '0', '0', 'admin', '2018-03-16 11:33:00', 'lisi', '2021-06-11 07:07:06', '普通角色');
INSERT INTO `t_sys_role` VALUES (3, '测试', 'test', 1, '4', '0', '0', 'admin', '2021-05-26 22:59:59', 'admin', '2021-08-03 22:33:33', '测试');

-- N13 菜单权限: 角色菜单关系表 动态路由
DROP TABLE IF EXISTS `t_sys_role_menu`;
CREATE TABLE `t_sys_role_menu`  (
  `role_id` int(20) NOT NULL COMMENT '角色ID',
  `menu_id` int(20) NOT NULL COMMENT '菜单ID',
  PRIMARY KEY (`role_id`, `menu_id`) USING BTREE
)ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '角色和菜单关联表';

-- N14 权限操作: 角色权限表 控制页面上的按钮是否有效
-- 目前提供两种方式：sys_btn_perm:
-- 1)动态路由 每个角色，针对每个页面进行定义 应保存不支持的操作
-- 2)静态、动态路由 每个角色，针对若干页面进行定义。
-- 记录各式：角色id,MENUID:[80,802]:PERM:70,71
DROP TABLE IF EXISTS `t_sys_role_perm`;
CREATE TABLE `t_sys_role_perm` (
  `role_id` int(20) NOT NULL COMMENT '角色ID',
  `menu_id` varchar(300) NOT NULL DEFAULT '0' COMMENT '菜单ID', --  0表示所有页面
  `component` varchar(50) COMMENT '路径(APP端用)', -- 菜单权限(针对APP显示特定菜单) 
  `perm` varchar(300) DEFAULT '0' COMMENT '按钮权限', -- 0表示所有操作
  PRIMARY KEY (`role_id`, `menu_id`) USING BTREE
)ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '角色权限表';
INSERT INTO `t_sys_role_perm` VALUES ('1', '0', '*',''); -- 管理员
INSERT INTO `t_sys_role_perm` VALUES ('5', '0', '/worker',''); -- 工人报修
INSERT INTO `t_sys_role_perm` VALUES ('6', '0', '/workerRepair',''); -- 维护人员


-- N15 备用：角色部门关系表
DROP TABLE IF EXISTS `t_sys_role_dept`;
CREATE TABLE `t_sys_role_dept`  (
  `role_id` int(20) NOT NULL COMMENT '角色ID',
  `dept_id` int(20) NOT NULL COMMENT '部门ID',
  PRIMARY KEY (`role_id`, `dept_id`) USING BTREE
)ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '角色和部门关联表';
	

-- N16 系统用户信息表  默认用户admin 密码：123456 ATV::sys_user
DROP TABLE IF EXISTS `t_sys_user`;
CREATE TABLE `t_sys_user`  (
  `user_id` int(20) NOT NULL AUTO_INCREMENT COMMENT '用户ID',
  `dept_id` int(20) COMMENT '部门ID(具有唯一性)',
  `role_ids` varchar(120) COMMENT '角色ID(默认单角色)',  -- 多角色比如[1,2]：用,分割role_id
  `post_ids` varchar(120) COMMENT '岗位ID(,分割post_id)',
  `user_name` varchar(30) COMMENT '登录账号(具有唯一性)',
  `nick_name` varchar(30) COMMENT '用户昵称(真实姓名)',
  `email` varchar(50) DEFAULT '' COMMENT '用户邮箱',
  `phonenumber` varchar(11) DEFAULT '' COMMENT '手机号码',
  `sex` char(1) DEFAULT '0' COMMENT '用户性别', -- 0男 1女 2未知
  `avatar` varchar(100) DEFAULT '' COMMENT '头像地址',  -- 命名规则
  `password` varchar(80) DEFAULT '' COMMENT '密码',
  `status` char(1) DEFAULT '0' COMMENT '帐号状态', -- 0正常 1停用
  `del_flag` char(1) DEFAULT '0' COMMENT '删除标志', -- 0代表存在 1代表删除
  `login_ip` varchar(50) DEFAULT '' COMMENT '最后登陆IP(或手机唯一标志)',
  `login_date` datetime(0) COMMENT '最后登陆时间',
  `create_by` varchar(64) DEFAULT '' COMMENT '创建者',
  `create_time` datetime(0) COMMENT '创建时间',
  `update_by` varchar(64) DEFAULT '' COMMENT '更新者',
  `update_time` datetime(0) COMMENT '更新时间',
  `remark` varchar(150) COMMENT '备注',
  `salt` varchar(45) COMMENT 'md5密码盐',
  `client_id` varchar(64) COMMENT '设备ID',
  `rel_tenant_ids` varchar(100) COMMENT '多租户标识',
   PRIMARY KEY (`user_id`) USING BTREE,
   UNIQUE INDEX `uniq_sys_user_username` (`user_name`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '用户信息表';
INSERT INTO `t_sys_user` VALUES (2, 101,'[2]','', 'mooding', '张三', 'mooding@qq.com', '15666666666', '2', 'IMG_3237.JPG', '$2a$10$KH71UqwS4z3r/XH/2phgqOloT3DLkKiuENK7z5uYSjXhnJkCrZcCC', '0', '0', '101.224.170.163', '2021-12-29 22:00:43', 'admin', '2018-03-16 11:33:00', 'mooding', '2021-12-29 22:01:24', '测试员','','','');
INSERT INTO `t_sys_user` VALUES (11, 102,'','', 'zhangsan', 'zhangsan', 'loveyou@qq.com', '15138032719', '2', '', '', '0', '0', '116.235.167.214', '2021-10-15 19:37:33', 'admin', NULL, 'admin', '2021-12-03 21:22:37', '老大','','','');
INSERT INTO `t_sys_user` VALUES (12, 108,'','', 'lisi', '李四', '110@qq.com', '15138032718', '1', '', 'IMG_3237.JPG', '0', '0', '116.235.167.214', '2021-10-15 19:37:45', 'admin', '2021-05-27 20:38:59', 'admin', '2021-11-20 21:48:43', 'test','','','');
INSERT INTO `t_sys_user` VALUES (1, 103,'[1,2]','', 'admin', '张三', 'mooding@163.com', '15888888888', '1', 'IMG_3237.JPG','$2a$10$KH71UqwS4z3r/XH/2phgqOloT3DLkKiuENK7z5uYSjXhnJkCrZcCC', '0', '0', '101.224.170.163', '2021-12-29 21:58:06', 'admin', '2018-03-16 11:33:00', 'admin', '2021-12-29 21:58:06', '管理员','','','');

INSERT INTO `t_sys_user` VALUES (5, 103,'[5]','', 'admin5', '张三', 'bjtu@163.com', '15888888888', '1', 'IMG_3237.JPG','$2a$10$KH71UqwS4z3r/XH/2phgqOloT3DLkKiuENK7z5uYSjXhnJkCrZcCC', '0', '0', '101.224.170.163', '2021-12-29 21:58:06', 'admin', '2018-03-16 11:33:00', 'admin5', '2021-12-29 21:58:06', '管理员','','','');
INSERT INTO `t_sys_user` VALUES (6, 103,'[6]','', 'admin6', '张三', 'bjtu@163.com', '15888888888', '1', 'IMG_3237.JPG','$2a$10$KH71UqwS4z3r/XH/2phgqOloT3DLkKiuENK7z5uYSjXhnJkCrZcCC', '0', '0', '101.224.170.163', '2021-12-29 21:58:06', 'admin', '2018-03-16 11:33:00', 'admin6', '2021-12-29 21:58:06', '管理员','','','');
-- 二选一
-- INSERT INTO `t_sys_user` VALUES (1, 103,'[1]','', 'admin', '张三', 'mooding@163.com', '15888888888', '1', 'IMG_3237.JPG','$2a$10$KH71UqwS4z3r/XH/2phgqOloT3DLkKiuENK7z5uYSjXhnJkCrZcCC', '0', '0', '101.224.170.163', '2021-12-29 21:58:06', 'admin', '2018-03-16 11:33:00', 'admin', '2021-12-29 21:58:06', '管理员','','','');
-- INSERT INTO `t_sys_user` VALUES (1, 103,'[1,2]','', 'admin', '张三', 'mooding@163.com', '15888888888', '1', 'IMG_3237.JPG','$2a$10$KH71UqwS4z3r/XH/2phgqOloT3DLkKiuENK7z5uYSjXhnJkCrZcCC', '0', '0', '101.224.170.163', '2021-12-29 21:58:06', 'admin', '2018-03-16 11:33:00', 'admin', '2021-12-29 21:58:06', '管理员','','','');


-- N17 系统通知公告(首页)  
DROP TABLE IF EXISTS `t_sys_notice`;
CREATE TABLE `t_sys_notice` (
   id varchar(50) NOT NULL COMMENT '新闻ID（自增主键）',
  `news_title` varchar(100) NOT NULL COMMENT '新闻标题',
  `news_content` longtext COMMENT '新闻内容',
  `news_content_wang` longtext COMMENT '新闻内容，编辑器格式',
  `news_author` varchar(50) DEFAULT NULL COMMENT '作者名称',
  `news_publisher` varchar(50) COMMENT '新闻发布者',
  `news_date` datetime DEFAULT NULL COMMENT '新闻发表时间',
  `cover_image` tinyint(2) DEFAULT '0' COMMENT '是否自定义封面图（1：自定义；2：默认）',
  `news_imgpath` varchar(80) DEFAULT NULL COMMENT '新闻封面图片路径',
  `news_details` varchar(80) DEFAULT NULL COMMENT '新闻详细内容跳转链接',
  `news_views` int DEFAULT '10' COMMENT '新闻浏览量',
  `news_type` tinyint(2) DEFAULT '0' COMMENT '新闻类型(0:公司新闻；1:行业资讯；2:通知公告)',
  `is_front` tinyint(2) DEFAULT '0' COMMENT '是否是首页新闻（1则代表是，0则代表不是）',
  `is_top` tinyint(2) DEFAULT '0' COMMENT '是否是置顶新闻（1：置顶；0：不置顶）',
  `is_outlink` tinyint(2) DEFAULT '0' COMMENT '是否是外部链接（0：不是外部链接；1：是外部链接）',
  `front_num` int DEFAULT '0' COMMENT '首页新闻的顺序',
   status tinyint(2) NOT NULL DEFAULT '1' COMMENT '新闻状态(0：草稿，1：已发布)',
  `create_by` varchar(64) DEFAULT '' COMMENT '创建者',
  `create_time` datetime(0) COMMENT '创建时间',
  `update_by` varchar(64) DEFAULT '' COMMENT '更新者',
  `update_time` datetime(0) COMMENT '更新时间',
  `remark` varchar(80) DEFAULT '' COMMENT '备注信息',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT='新闻表';

-- N18 系统通告表(推送消息): 原：sys_announcement
-- 与N17 系统通知公告(首页) 合并？
DROP TABLE IF EXISTS `t_sys_announcement`;
CREATE TABLE `t_sys_announcement`  (
  `id` varchar(32) NOT NULL,
  `titile` varchar(100) COMMENT '标题',
  `msg_content` text COMMENT '内容',
  `start_time` datetime(0) COMMENT '开始时间',
  `end_time` datetime(0) COMMENT '结束时间',
  `sender` varchar(100) COMMENT '发布人',
  `priority` varchar(255) COMMENT '优先级（L低，M中，H高）',
  `msg_category` char(1) DEFAULT '2' COMMENT '消息类型1:通知公告2:系统消息',
  `msg_type` varchar(10) COMMENT '通告对象类型（USER:指定用户，ALL:全体用户）',
  `send_status` char(1) COMMENT '发布状态（0未发布，1已发布，2已撤销）',
  `send_time` datetime(0) COMMENT '发布时间',
  `cancel_time` datetime(0) COMMENT '撤销时间',
  `del_flag` char(1) COMMENT '删除状态（0，正常，1已删除）',
  `bus_type` varchar(20) COMMENT '业务类型(email:邮件 bpm:流程)',
  `bus_id` varchar(50) COMMENT '业务id',
  `open_type` varchar(20) COMMENT '打开方式(组件：component 路由：url)',
  `open_page` varchar(255) COMMENT '组件/路由 地址',
  `create_by` varchar(32) COMMENT '创建人',
  `create_time` datetime(0) COMMENT '创建时间',
  `update_by` varchar(32) COMMENT '更新人',
  `update_time` datetime(0) COMMENT '更新时间',
  `user_ids` text COMMENT '指定用户',
  `msg_abstract` text COMMENT '摘要',
  PRIMARY KEY (`id`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '系统通告表' ;
INSERT INTO `t_sys_announcement` VALUES ('1256486502931722242', 'JeecgBoot 2.2.0版本发布', '<h5 id=\"h5_5\" style=\"box-sizing: inherit; font-family: \'PingFang SC\', \'Helvetica Neue\', \'Microsoft YaHei UI\', \'Microsoft YaHei\', \'Noto Sans CJK SC\', Sathu, EucrosiaUPC, Arial, Helvetica, sans-serif; line-height: 1.8; margin: 22px 0px 16px; padding: 0px; font-size: 18px; border: none; color: #333333; background-color: #ffffff;\">升级不兼容</h5>\n<ul style=\"box-sizing: inherit; margin: 0px 0px 20px; padding: 0px 0px 0px 20px; color: #333333; font-family: -apple-system, BlinkMacSystemFont, \'Apple Color Emoji\', \'Segoe UI Emoji\', \'Segoe UI Symbol\', \'Segoe UI\', \'PingFang SC\', \'Hiragino Sans GB\', \'Microsoft YaHei\', \'Helvetica Neue\', Helvetica, Arial, sans-serif; font-size: 16px; background-color: #ffffff;\">\n<li style=\"box-sizing: inherit; line-height: 1.875em; margin-top: 0px;\">1、升级 ant-design-vue 到最新版本 1.5.2，会导致使用disabled的组件不可用（需要全局替换disabled 为readOnly；另外原来readOnly不允许全小写）</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">2、JSearchSelectTag.vue行编辑的popup传参做了修改，需要全局替换 orgFieldse 为 orgFields</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">3、 项目删除了sass，全部换成 less，其中涉及/deep/语法得改造</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em; margin-bottom: 0px;\">4、 Online表单的填值规则用法修改了&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" href=\"http://doc.jeecg.com/1630378\">文档</a></li>\n</ul>\n<h5 id=\"h5_6\" style=\"box-sizing: inherit; font-family: \'PingFang SC\', \'Helvetica Neue\', \'Microsoft YaHei UI\', \'Microsoft YaHei\', \'Noto Sans CJK SC\', Sathu, EucrosiaUPC, Arial, Helvetica, sans-serif; line-height: 1.8; margin: 22px 0px 16px; padding: 0px; font-size: 18px; border: none; color: #333333; background-color: #ffffff;\">较大改造</h5>\n<ul style=\"box-sizing: inherit; margin: 0px 0px 20px; padding: 0px 0px 0px 20px; color: #333333; font-family: -apple-system, BlinkMacSystemFont, \'Apple Color Emoji\', \'Segoe UI Emoji\', \'Segoe UI Symbol\', \'Segoe UI\', \'PingFang SC\', \'Hiragino Sans GB\', \'Microsoft YaHei\', \'Helvetica Neue\', Helvetica, Arial, sans-serif; font-size: 16px; background-color: #ffffff;\">\n<li style=\"box-sizing: inherit; line-height: 1.875em; margin-top: 0px;\">登录的时候一次性加载系统字典，进行前端缓存，减少ajax重复请求</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">修改定时任务quartz配置</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">logback-spring.xml配置不按照日期和大小生成新日志文件问题修复</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">vue-cli3 编译打包进行zip压缩优化</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">根据index.html页面window._CONFIG[\'domianURL\']，指定 axios的 baseURL（所以vue的代理没有用了）</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">JEditableTable重构，新增获取值时应该把临时id去掉、行编辑修改不直接更改数据源的ID</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em; margin-bottom: 0px;\">Online模块重点优化升级、Online代码生成器重点优化升级</li>\n</ul>\n<h5 id=\"h5_7\" style=\"box-sizing: inherit; font-family: \'PingFang SC\', \'Helvetica Neue\', \'Microsoft YaHei UI\', \'Microsoft YaHei\', \'Noto Sans CJK SC\', Sathu, EucrosiaUPC, Arial, Helvetica, sans-serif; line-height: 1.8; margin: 22px 0px 16px; padding: 0px; font-size: 18px; border: none; color: #333333; background-color: #ffffff;\">ONLINE升级</h5>\n<ul style=\"box-sizing: inherit; margin: 0px 0px 20px; padding: 0px 0px 0px 20px; color: #333333; font-family: -apple-system, BlinkMacSystemFont, \'Apple Color Emoji\', \'Segoe UI Emoji\', \'Segoe UI Symbol\', \'Segoe UI\', \'PingFang SC\', \'Hiragino Sans GB\', \'Microsoft YaHei\', \'Helvetica Neue\', Helvetica, Arial, sans-serif; font-size: 16px; background-color: #ffffff;\">\n<li style=\"box-sizing: inherit; line-height: 1.875em; margin-top: 0px;\">Online访问权限控制实现，如果online表单配置了菜单，则用户需要授权才能访问此表单</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单新增组件： 省市区地域组件、开关组件、三级联动(级联下拉)组件、markdown组件</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单弹窗宽度，根据不同的列数显示不同的宽度，实现更好的效果</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online 树表单功能一系列优化，比如数据列新增添加下级按钮功能、添加下级不自动展开列表等问题</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online JS增强编辑器支持放大，同时JS代码变更，增加版本记忆功能，方便追踪历史</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online JS增强支持lodash工具</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online控件 默认值表达式功能&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" href=\"http://doc.jeecg.com/1630378\">文档</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online 增加Excel导入导出转换器配置，针对各种控件的导入导出做了统一处理</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online 列表字段排序支持配置</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online同步数据库，数据库兼容性优化（mysql、oracle11g、SqlServer2017）</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online功能列表，高级查询优化，丰富控件的支持（新增：用户选择、部门选择、时间等）</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单配置，类型Text、date、Blob的字段长度默认设置为0</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online自定义button，支持位置自定义（支持右侧和下方）</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单列表，列表显示那些字段，支持用户个性化设置</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单删除和移除逻辑优化，防止未同步情况下删除报错</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online上传逻辑重构，针对单表、一对多行操作，都支持图片和文件上传，同时支持三种模式（minio、阿里云、本地存储）</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online上传逻辑重构， 行编辑模式，支持图片顺序修改</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online上传逻辑重构，在列表页面支持图片和文件的展示</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单索引同步逻辑重构，同时支持选择多个字段，组合索引设置</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单 excel导出实现数据权限逻辑</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单列表强制列不换行，自动出现超出滚定条，支持移动端自适应（单表、树和ERP等模型都已修改）</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单管理列表，回车查询和列表左右拖动移动自适应问题</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单附表序号必填</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表名加校验不能全是数字，自动trim空格</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online 一对多tab必填校验不通过，添加友好提醒</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单控件默认值示例</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online一对多内嵌子表风格，高级查询不好用，过滤不了数据处理</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单 自定义校验规则正则，录入不成功问题处理</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单设置不分页，排序后又分页了问题处理</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单，下拉搜索组件，表字典配置加条件，下拉值出不来问题处理</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单，一对一子表，下拉选择、时间等组件被遮挡问题处理</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online表单树模型和一对多ERP模板支持href</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online 报表支持上下文变量表达式</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online 报表实现数据权限逻辑</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online 报表新增拖动字段顺序</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online 报表列表，sql过长截取显示</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">Online 报表，popup回车查询</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em; margin-bottom: 0px;\">Online表单auto页面弹窗改成JModule；online表单移动自适应；</li>\n</ul>\n<h5 id=\"h5_8\" style=\"box-sizing: inherit; font-family: \'PingFang SC\', \'Helvetica Neue\', \'Microsoft YaHei UI\', \'Microsoft YaHei\', \'Noto Sans CJK SC\', Sathu, EucrosiaUPC, Arial, Helvetica, sans-serif; line-height: 1.8; margin: 22px 0px 16px; padding: 0px; font-size: 18px; border: none; color: #333333; background-color: #ffffff;\">Online代码生成器升级</h5>\n<blockquote style=\"box-sizing: inherit; position: relative; margin: 0px 0px 20px; padding: 20px; background-color: #f6f6f6; border-left: 6px solid #e6e6e6; word-break: break-word; color: #333333; font-family: -apple-system, BlinkMacSystemFont, \'Apple Color Emoji\', \'Segoe UI Emoji\', \'Segoe UI Symbol\', \'Segoe UI\', \'PingFang SC\', \'Hiragino Sans GB\', \'Microsoft YaHei\', \'Helvetica Neue\', Helvetica, Arial, sans-serif; font-size: 16px;\">\n<p style=\"box-sizing: inherit; margin: 0px; line-height: inherit;\">后期重点维护ONLINE模式的代码生成器，GUI模式逐步弃用。</p>\n</blockquote>\n<ul style=\"box-sizing: inherit; margin: 0px 0px 20px; padding: 0px 0px 0px 20px; color: #333333; font-family: -apple-system, BlinkMacSystemFont, \'Apple Color Emoji\', \'Segoe UI Emoji\', \'Segoe UI Symbol\', \'Segoe UI\', \'PingFang SC\', \'Hiragino Sans GB\', \'Microsoft YaHei\', \'Helvetica Neue\', Helvetica, Arial, sans-serif; font-size: 16px; background-color: #ffffff;\">\n<li style=\"box-sizing: inherit; line-height: 1.875em; margin-top: 0px;\">新增内嵌Table代码生成器模板</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">升级代码生成器依赖，支持返回生成结果&amp;支持服务器端生成代码config</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">代码生成器列表，列表自适应优化，支持移动效果（强制列不换行，不过有弊端，<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" href=\"http://doc.jeecg.com/1607183\">见文档</a>）</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">实体生成，自动带着swagger 注解（支持单表、一对多等模型）</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">代码生成器，列表移动自适应样式修改</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">生成后的代码导入导出bug修复</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">online树代码生成器模板，直接删除一级分类会导致列表乱</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">ERP代码生成器模板升级，子表支持导入导出</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">ERP风格代码生成器模板问题（导入导入路径错误、swagger注解问题）</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">代码生成器，校验为空的话不生成，简化生成的代码</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">代码生成器，支持子表校验生成</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">树形列表代码生成器优化，添加子节点</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">修复bug： online字典未配置，代码生成器报错问题</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">生成的实体字段excel注解，针对系统标准字段，创建人、创建时间等字段不生成</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">生成的表单弹窗统一更换为j-modal，支持放大缩小全屏</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">代码生成，popup参数大小写 驼峰问题修复</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">生成的表单控件类型更丰富，新支持控件：markdown、省市区地域、密码、下拉搜索</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em; margin-bottom: 0px;\">生成的列表查询区域，控件支持加强，新支持控件：省市区地域、下拉搜索、popup、选择用户、选择部门</li>\n</ul>\n<h5 id=\"h5_9\" style=\"box-sizing: inherit; font-family: \'PingFang SC\', \'Helvetica Neue\', \'Microsoft YaHei UI\', \'Microsoft YaHei\', \'Noto Sans CJK SC\', Sathu, EucrosiaUPC, Arial, Helvetica, sans-serif; line-height: 1.8; margin: 22px 0px 16px; padding: 0px; font-size: 18px; border: none; color: #333333; background-color: #ffffff;\">平台基础升级</h5>\n<ul style=\"box-sizing: inherit; margin: 0px 0px 20px; padding: 0px 0px 0px 20px; color: #333333; font-family: -apple-system, BlinkMacSystemFont, \'Apple Color Emoji\', \'Segoe UI Emoji\', \'Segoe UI Symbol\', \'Segoe UI\', \'PingFang SC\', \'Hiragino Sans GB\', \'Microsoft YaHei\', \'Helvetica Neue\', Helvetica, Arial, sans-serif; font-size: 16px; background-color: #ffffff;\">\n<li style=\"box-sizing: inherit; line-height: 1.875em; margin-top: 0px;\">针对auto隐藏路由菜单，实现自动授权，简化online菜单的授权</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">IDE中找不到sun.misc.BASE64Encoder jar包问题</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">删掉作废获取所有用户角色列表接口，接口有性能问题</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">在我的消息里点击已读以后，首页上面的通知红点并没有同步更改问题修复</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">用户与部门取消关联删除关联关系bug修改</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">部门导入未刷新redis缓存</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">修复SysPermissionMapper.xml 的SQL语句不兼容SQLServer的问题</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">在线动态数据源代码重构</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">【二级管理员】部门修改、删除权限时关联删除部门角色数据</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">【二级管理员】我的部门，选中部门只能看当前部门下的角色</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">我的消息支持模糊查询</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">常用示例-对象存储文件上传优化</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">用户相关功能，sql写法不兼容其他数据库问题修复</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">图片上传组件修改</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">导入功能优化，提示准确导入成功失败信息，涉及功能 用户、角色、部门、字典、定时任务等</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">增加生产环境禁用swagger-ui配置</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">第三方登录代码集成</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">系统公告优化</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">MockController 返回json数据，utf-8格式化，防止中文乱码</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">部门删除后删除部门角色等关联</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em; margin-bottom: 0px;\">分类字典支持选择数据，添加下级分类</li>\n</ul>\n<h5 id=\"h5_10\" style=\"box-sizing: inherit; font-family: \'PingFang SC\', \'Helvetica Neue\', \'Microsoft YaHei UI\', \'Microsoft YaHei\', \'Noto Sans CJK SC\', Sathu, EucrosiaUPC, Arial, Helvetica, sans-serif; line-height: 1.8; margin: 22px 0px 16px; padding: 0px; font-size: 18px; border: none; color: #333333; background-color: #ffffff;\">UI组件升级</h5>\n<ul style=\"box-sizing: inherit; margin: 0px 0px 20px; padding: 0px 0px 0px 20px; color: #333333; font-family: -apple-system, BlinkMacSystemFont, \'Apple Color Emoji\', \'Segoe UI Emoji\', \'Segoe UI Symbol\', \'Segoe UI\', \'PingFang SC\', \'Hiragino Sans GB\', \'Microsoft YaHei\', \'Helvetica Neue\', Helvetica, Arial, sans-serif; font-size: 16px; background-color: #ffffff;\">\n<li style=\"box-sizing: inherit; line-height: 1.875em; margin-top: 0px;\">升级 ant-design-vue 为最新版本 1.5.2</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">登录验证码获取失败的时候，不显示空白</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">升级 ant-design-vue，JModal切换全屏按钮因ant升级错位问题修复</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">简化 j-modal 的相关代码</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">封装常用组件示例；JInput新增当 type 变化的时候重新计算值</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">markdown集成</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">富文本支持minio上传</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">选择用户组件列表样式错位；Ellipsis.vue 优化</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">JModal移动端全屏效果</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">高级查询只有在 in 模式下才能多选</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">自定义选择用户组件JSelectMultiUser的宽度和截取显示长度</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">行编辑 JEditableTable移动自适应效果；</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">JEditableTable重构 行编辑新增合计功能</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">JEditableTable重构 行编辑Popup请求数量过多（加缓存机制）</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">JEditableTable重构,行编辑InputNumber扩展支持输入小数和负数</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">JEditableTable.vue disable模式禁用添加删除按钮</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">JEditableTable行编辑， popup返回值，时间赋值有问题</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">JEditableTable行编辑，日期控件显示错位问题</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">高级查询移动自适应效果；</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">高级查询保存的同时也要保存匹配方式（支持and or）</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">高级查询混入统一修改</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">高级查询选择用户组件时，点开用户弹框不选择用户，直接关掉用户弹框，再次点击弹框时，无法点开问题</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">高级查询组件支持，新支持组件，选择人、选择部门、popup、时间等</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">JPopup支持多选</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">JPopup返回值null问题</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">JPopup列表数据不刷新问题</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">JSearchSelectTag.vue匹配不上数字值问题</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">JSelectUserByDep 根据部门选择人组件bug</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">JTreeDict.vue 分类字典组件 清空不了问题</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">常用示例，新增搜索下拉 JSearchSelectTag.vue例子</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">常用示例，新增如何关闭当前页面例子</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">常用示例，省市区三级联动的例子</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">常用示例，增加字典下拉多选组件示例</li>\n<li style=\"box-sizing: inherit; line-height: 1.875em; margin-bottom: 0px;\">示例demo，关于图片的做统一优化</li>\n</ul>\n<h5 id=\"h5_11\" style=\"box-sizing: inherit; font-family: \'PingFang SC\', \'Helvetica Neue\', \'Microsoft YaHei UI\', \'Microsoft YaHei\', \'Noto Sans CJK SC\', Sathu, EucrosiaUPC, Arial, Helvetica, sans-serif; line-height: 1.8; margin: 22px 0px 16px; padding: 0px; font-size: 18px; border: none; color: #333333; background-color: #ffffff;\">Issues处理</h5>\n<ul style=\"box-sizing: inherit; margin: 0px 0px 20px; padding: 0px 0px 0px 20px; color: #333333; font-family: -apple-system, BlinkMacSystemFont, \'Apple Color Emoji\', \'Segoe UI Emoji\', \'Segoe UI Symbol\', \'Segoe UI\', \'PingFang SC\', \'Hiragino Sans GB\', \'Microsoft YaHei\', \'Helvetica Neue\', Helvetica, Arial, sans-serif; font-size: 16px; background-color: #ffffff;\">\n<li style=\"box-sizing: inherit; line-height: 1.875em; margin-top: 0px;\">online form表单项能否支持配置有级联关系的内容&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#948\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/948\">#948</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">全局拦截异常错误的提醒文字&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#768\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/768\">#768</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">建议：城市级联选择组件&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#905\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/905\">#905</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">表单配置中检验字段配置href，报表中该配置字段无法看到链接&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#961\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/961\">#961</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">SysUserMapper.xml查询脚本数据兼容问题&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#962\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/962\">#962</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">在线表单 提示 ButtonExpHandler is not defined&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#957\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/957\">#957</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">online表单中添加自定义按钮无法显示&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#973\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/973\">#973</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">online报表不支持按照登录用户过滤数据&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#934\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/934\">#934</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">点击新增或者编辑按钮弹框宽度的设置问题&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#974\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/974\">#974</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">online报表配置中使用系统参数，报错&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"I1AWNM\" href=\"https://gitee.com/jeecg/jeecg-boot/issues/I1AWNM\">I1AWNM</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">online增强SQL变量取数为null,#{sys.sys_date} 还有#{sys.sys_time}&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#999\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/999\">#999</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">一个主表，多个付表。1对多。保存后，某些从表行项目内容保存成功，单重新打开数据表现丢失。&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#997\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/997\">#997</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">/online/cgform/api/exportXls/{code}接口问题&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1012\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1012\">#1012</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">打成jar包，访问不到代码生成器模板&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1010\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1010\">#1010</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">online表单开发，打包后模板找不到&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#865\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/865\">#865</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">已生成的代码，子表最下下行添加合计行底部固定问题&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#936\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/936\">#936</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">下拉搜索框在编辑时下拉name无法带过来&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#971\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/971\">#971</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">建议autoPoi升级，优化数据返回List Map格式下的复合表头导出excel的体验&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#873\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/873\">#873</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">配置 Href 跳转，并没有显示为超链接&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1020\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1020\">#1020</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">首次打开一对多表单，JEditableTable的addDefaultRowNum属性不生效&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1003\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1003\">#1003</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">autopoi 双表头问题&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#862\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/862\">#862</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">应该是权限管理bug&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#110\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/110\">#110</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">在线 的online报表报错&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1029\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1029\">#1029</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">online代码生成器树形表单父节点字段名称问题&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"issues/I1ABGV \" href=\"https://gitee.com/jeecg/jeecg-boot/issues/I1ABGV\">issues/I1ABGV</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">点击j-popup,里面有数据，如果选择同名的数据，文本框为空&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1044\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1044\">#1044</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">关于table列宽可拖动问题&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1054\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1054\">#1054</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">单表数据导出多表头 auto 的Excel注解 groupName属性<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\" #1053\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1053\">&nbsp;#1053</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">在线报表和在线online的问题&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1030\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1030\">#1030</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">J-pop组件&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1043\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1043\">#1043</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">2.1.4 日志管理---没有记录查询接口参数&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1070\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1070\">#1070</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">服务器日志超过设置的MaxFileSize时&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1130\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1130\">#1130</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">online 表单中，在 附表中存在上传图片组件，点击上传图片并提交，显示提交失败&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1074\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1074\">#1074</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">online表单js增强修改从表控件值问题&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1051\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1051\">#1051</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">重复代理问题&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#994\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/994\">#994</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">系统使用的日志包有问题&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#887\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/887\">#887</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">使用sqlserver数据库,用户管理查询出错&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1140\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1140\">#1140</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">主表和从表 一对一关系表单 TypeError: Cannot read property \'getAll\' of undefined&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"#1129\" href=\"https://github.com/zhangdaiscott/jeecg-boot/issues/1129\">#1129</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">ERP表单附表数据编辑后消失&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"issues/I1DN3B\" href=\"https://gitee.com/jeecg/jeecg-boot/issues/I1DN3B\">issues/I1DN3B</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">多表的erp模式生成的子表，表单没有导入导出功能&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"issues/I1BB2U\" href=\"https://gitee.com/jeecg/jeecg-boot/issues/I1BB2U\">issues/I1BB2U</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">用edge打开首页，整个页面动不了，控制台console打出很多错误&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"issues/I189B5\" href=\"https://gitee.com/jeecg/jeecg-boot/issues/I189B5\">issues/I189B5</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">官方的快速开发平台主子表对IE11不兼容&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"issues/I17LEE\" href=\"https://gitee.com/jeecg/jeecg-boot/issues/I17LEE\">issues/I17LEE</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em;\">360兼容模式 修改为你说的js之后；单表的数据兼容可以；主子表的不加载；错误如下&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"issues/I17H8L\" href=\"https://gitee.com/jeecg/jeecg-boot/issues/I17H8L\">issues/I17H8L</a></li>\n<li style=\"box-sizing: inherit; line-height: 1.875em; margin-bottom: 0px;\">Online表单开发，点击&ldquo;新增&rdquo;按钮，是否树：选择是，页面控制台报错&nbsp;<a style=\"box-sizing: inherit; background-color: transparent; color: #4183c4; text-decoration-line: none;\" title=\"issues/I1BHXG\" href=\"https://gitee.com/jeecg/jeecg-boot/issues/I1BHXG\">issues/I1BHXG</a></li>\n</ul>', '2020-05-01 15:30:56', '2020-05-30 15:31:00', 'admin', 'H', '1', 'ALL', '1', '2020-05-02 15:31:44', NULL, '0', NULL, NULL, NULL, NULL, 'admin', '2020-05-02 15:31:41', 'admin', '2020-05-02 15:31:44', NULL, '重磅版本发布');
INSERT INTO `t_sys_announcement` VALUES ('1256486817286418434', '放假通知', '<p>放假通知</p>', '2020-05-02 15:32:38', '2020-05-30 15:32:40', 'admin', 'M', '1', 'USER', '1', '2020-05-02 15:32:59', NULL, '0', NULL, NULL, NULL, NULL, 'admin', '2020-05-02 15:32:56', 'admin', '2020-05-02 15:32:59', 'e9ca23d68d884d4ebb19d07889727dae,a75d45a015c44384a04449ee80dc3503,', '放假通知');
INSERT INTO `t_sys_announcement` VALUES ('1256529336858685441', '1', '<p>22</p>', '2020-05-02 18:21:41', '2020-05-30 18:21:44', 'admin', 'M', '1', 'ALL', '2', '2020-05-02 18:21:57', '2020-05-02 18:22:07', '0', NULL, NULL, NULL, NULL, 'admin', '2020-05-02 18:21:54', 'admin', '2020-05-02 18:22:07', NULL, '22');
INSERT INTO `t_sys_announcement` VALUES ('1b714f8ebc3cc33f8b4f906103b6a18d', '5467567', NULL, NULL, NULL, 'admin', NULL, '2', NULL, '1', '2019-03-30 12:40:38', NULL, '1', NULL, NULL, NULL, NULL, 'admin', '2019-02-26 17:23:26', 'admin', '2020-05-02 15:30:42', NULL, NULL);
INSERT INTO `t_sys_announcement` VALUES ('3d11237ccdf62450d20bb8abdb331178', '111222', NULL, NULL, NULL, NULL, NULL, '2', NULL, '0', NULL, NULL, '1', NULL, NULL, NULL, NULL, 'admin', '2019-03-29 17:19:47', 'admin', '2019-03-29 17:19:50', NULL, NULL);
INSERT INTO `t_sys_announcement` VALUES ('7ef04e95f8de030b1d5f7a9144090dc6', '111', NULL, '2019-02-06 17:28:10', '2019-03-08 17:28:11', NULL, NULL, '2', NULL, '0', NULL, NULL, '1', NULL, NULL, NULL, NULL, 'admin', '2019-02-26 17:28:17', 'admin', '2019-03-26 19:59:49', NULL, NULL);
INSERT INTO `t_sys_announcement` VALUES ('93a9060a1c20e4bf98b3f768a02c2ff9', '111', '111', '2019-02-06 17:20:17', '2019-02-21 17:20:20', 'admin', 'M', '2', 'ALL', '1', '2019-02-26 17:24:29', NULL, '1', NULL, NULL, NULL, NULL, 'admin', '2019-02-26 17:16:26', 'admin', '2020-05-02 15:30:42', NULL, NULL);
INSERT INTO `t_sys_announcement` VALUES ('de1dc57f31037079e1e55c8347fe6ef7', '222', '2222', '2019-02-06 17:28:26', '2019-02-23 17:28:28', 'admin', 'M', '2', 'ALL', '1', '2019-03-29 17:19:56', NULL, '1', NULL, NULL, NULL, NULL, 'admin', '2019-02-26 17:28:36', 'admin', '2019-02-26 17:28:40', NULL, NULL);
INSERT INTO `t_sys_announcement` VALUES ('e52f3eb6215f139cb2224c52517af3bd', '334', '334', NULL, NULL, NULL, NULL, '2', NULL, '0', NULL, NULL, '1', NULL, NULL, NULL, NULL, 'admin', '2019-03-30 12:40:28', 'admin', '2019-03-30 12:40:32', NULL, NULL);

-- N19 用户通告阅读标记表: 原：sys_announcement_send
DROP TABLE IF EXISTS `t_sys_announcement_send`;
CREATE TABLE `t_sys_announcement_send`  (
  `id` varchar(32) NULL DEFAULT NULL,
  `annt_id` varchar(32) COMMENT '通告ID',
  `user_id` varchar(32) COMMENT '用户id',
  `read_flag` char(1) COMMENT '阅读状态（0未读，1已读）',
  `read_time` datetime(0) COMMENT '阅读时间',
  `create_by` varchar(32) COMMENT '创建人',
  `create_time` datetime(0) COMMENT '创建时间',
  `update_by` varchar(32) COMMENT '更新人',
  `update_time` datetime(0) COMMENT '更新时间'
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci COMMENT = '用户通告阅读标记表';
INSERT INTO `t_sys_announcement_send` VALUES ('646c0c405ec643d4dc4160db2446f8ff', '93a9060a1c20e4bf98b3f768a02c2ff9', 'e9ca23d68d884d4ebb19d07889727dae', '1', '2019-11-21 16:30:01', 'admin', '2019-05-17 11:50:56', 'admin', '2019-11-21 16:30:01');
INSERT INTO `t_sys_announcement_send` VALUES ('1197434450981543938', '93a9060a1c20e4bf98b3f768a02c2ff9', 'a75d45a015c44384a04449ee80dc3503', '0', NULL, 'jeecg', '2019-11-21 16:39:55', NULL, NULL);
INSERT INTO `t_sys_announcement_send` VALUES ('1256486817319972866', '1256486817286418434', 'e9ca23d68d884d4ebb19d07889727dae', '1', '2020-05-02 15:33:05', 'admin', '2020-05-02 15:32:56', 'admin', '2020-05-02 15:33:05');
INSERT INTO `t_sys_announcement_send` VALUES ('1256486817349332993', '1256486817286418434', 'a75d45a015c44384a04449ee80dc3503', '1', '2020-10-13 16:05:25', 'admin', '2020-05-02 15:32:56', 'jeecg', '2020-10-13 16:05:25');
INSERT INTO `t_sys_announcement_send` VALUES ('1256527099214278657', '1256486502931722242', 'e9ca23d68d884d4ebb19d07889727dae', '1', '2020-05-02 18:22:00', 'admin', '2020-05-02 18:12:59', 'admin', '2020-05-02 18:22:00');
INSERT INTO `t_sys_announcement_send` VALUES ('1260927781673484290', '1256486502931722242', 'a75d45a015c44384a04449ee80dc3503', '0', NULL, 'jeecg', '2020-05-14 21:39:45', NULL, NULL);

-- N20 消息模板: 原：sys_sms_template
DROP TABLE IF EXISTS `t_sys_sms_template`;
CREATE TABLE `t_sys_sms_template`  (
  `id` varchar(32) NOT NULL COMMENT '主键',
  `template_name` varchar(50) COMMENT '模板标题',
  `template_code` varchar(32) COMMENT '模板CODE',
  `template_type` varchar(1) COMMENT '模板类型：1短信 2邮件 3微信',
  `template_content` varchar(1000) COMMENT '模板内容',
  `template_test_json` varchar(1000) COMMENT '模板测试json',
  `create_time` datetime(0) COMMENT '创建日期',
  `create_by` varchar(32) COMMENT '创建人登录名称',
  `update_time` datetime(0) COMMENT '更新日期',
  `update_by` varchar(32) COMMENT '更新人登录名称',
  PRIMARY KEY (`id`) USING BTREE,
  UNIQUE INDEX `uniq_templatecode`(`template_code`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;
INSERT INTO `t_sys_sms_template` VALUES ('1199606397416775681', '系统消息通知', 'sys_ts_note', '4', '<h1>&nbsp; &nbsp; 系统通知</h1>\n<ul>\n<li>通知时间：&nbsp; ${ts_date}</li>\n<li>通知内容：&nbsp; ${ts_content}</li>\n</ul>', NULL, '2019-11-27 16:30:27', 'admin', '2019-11-27 19:36:50', 'admin');
INSERT INTO `t_sys_sms_template` VALUES ('1199615897335095298', '流程催办', 'bpm_cuiban', '4', '<h1>&nbsp; &nbsp;流程催办提醒</h1>\n<ul>\n<li>流程名称：&nbsp; ${bpm_name}</li>\n<li>催办任务：&nbsp; ${bpm_task}</li>\n<li>催办时间 :&nbsp; &nbsp; ${datetime}</li>\n<li>催办内容 :&nbsp; &nbsp; ${remark}</li>\n</ul>', NULL, '2019-11-27 17:08:12', 'admin', '2019-11-27 19:36:45', 'admin');
INSERT INTO `t_sys_sms_template` VALUES ('1199648914107625473', '流程办理超时提醒', 'bpm_chaoshi_tip', '4', '<h1>&nbsp; &nbsp;流程办理超时提醒</h1>\n<ul>\n<li>&nbsp; &nbsp;超时提醒信息：&nbsp; &nbsp; 您有待处理的超时任务，请尽快处理！</li>\n<li>&nbsp; &nbsp;超时任务标题：&nbsp; &nbsp; ${title}</li>\n<li>&nbsp; &nbsp;超时任务节点：&nbsp; &nbsp; ${task}</li>\n<li>&nbsp; &nbsp;任务处理人：&nbsp; &nbsp; &nbsp; &nbsp;${user}</li>\n<li>&nbsp; &nbsp;任务开始时间：&nbsp; &nbsp; ${time}</li>\n</ul>', NULL, '2019-11-27 19:19:24', 'admin', '2019-11-27 19:36:37', 'admin');
INSERT INTO `t_sys_sms_template` VALUES ('4028608164691b000164693108140003', '催办：${taskName}', 'SYS001', '3', '${userName}，您好！\r\n请前待办任务办理事项！${taskName}\r\n\r\n\r\n===========================\r\n此消息由系统发出', '{\r\n\"taskName\":\"HR审批\",\r\n\"userName\":\"admin\"\r\n}', '2018-07-05 14:46:18', 'admin', '2018-07-05 18:31:34', 'admin');


-- N21 消息 原：sys_sms
DROP TABLE IF EXISTS `t_sys_sms`;
CREATE TABLE `t_sys_sms`  (
  `id` varchar(32) NOT NULL COMMENT 'ID',
  `es_title` varchar(100) COMMENT '消息标题',
  `es_type` varchar(1) COMMENT '发送方式：1短信 2邮件 3微信',
  `es_receiver` varchar(50) COMMENT '接收人',
  `es_param` varchar(1000) COMMENT '发送所需参数Json格式',
  `es_content` longtext COMMENT '推送内容',
  `es_send_time` datetime(0) COMMENT '推送时间',
  `es_send_status` varchar(1) COMMENT '推送状态 0未推送 1推送成功 2推送失败 -1失败不再发送',
  `es_send_num` int(11) COMMENT '发送次数 超过5次不再发送',
  `es_result` varchar(255) COMMENT '推送失败原因',
  `remark` varchar(500) COMMENT '备注',
  `create_by` varchar(32) COMMENT '创建人登录名称',
  `create_time` datetime(0) COMMENT '创建日期',
  `update_by` varchar(32) COMMENT '更新人登录名称',
  `update_time` datetime(0) COMMENT '更新日期',
  PRIMARY KEY (`id`) USING BTREE,
  INDEX `index_type`(`es_type`) USING BTREE,
  INDEX `index_receiver`(`es_receiver`) USING BTREE,
  INDEX `index_sendtime`(`es_send_time`) USING BTREE,
  INDEX `index_status`(`es_send_status`) USING BTREE
) ENGINE=InnoDB DEFAULT CHARSET=utf8mb4 COLLATE=utf8mb4_unicode_ci;
INSERT INTO `t_sys_sms` VALUES ('402880e74dc2f361014dc2f8411e0001', '消息推送测试333', '2', '411944058@qq.com', NULL, '张三你好，你的订单4028d881436d514601436d521ae80165已付款!', '2015-06-05 17:06:01', '3', NULL, NULL, '认证失败错误的用户名或者密码', 'admin', '2015-06-05 17:05:59', 'admin', '2015-11-19 22:30:39');
INSERT INTO `t_sys_sms` VALUES ('402880ea533647b00153364e74770001', '发个问候', '3', 'admin', NULL, '你好', '2016-03-02 00:00:00', '2', NULL, NULL, NULL, 'admin', '2016-03-02 15:50:24', 'admin', '2018-07-05 19:53:01');
INSERT INTO `t_sys_sms` VALUES ('402880ee5a17e711015a17f3188e013f', '消息推送测试333', '2', '411944058@qq.com', NULL, '张三你好，你的订单4028d881436d514601436d521ae80165已付款!', NULL, '2', NULL, NULL, NULL, 'admin', '2017-02-07 17:41:31', 'admin', '2017-03-10 11:37:05');
INSERT INTO `t_sys_sms` VALUES ('402880f05ab649b4015ab64b9cd80012', '消息推送测试333', '2', '411944058@qq.com', NULL, '张三你好，你的订单4028d881436d514601436d521ae80165已付款!', '2017-11-16 15:58:15', '3', NULL, NULL, NULL, 'admin', '2017-03-10 11:38:13', 'admin', '2017-07-31 17:24:54');
INSERT INTO `t_sys_sms` VALUES ('402880f05ab7b035015ab7c4462c0004', '消息推送测试333', '2', '411944058@qq.com', NULL, '张三你好，你的订单4028d881436d514601436d521ae80165已付款!', '2017-11-16 15:58:15', '3', NULL, NULL, NULL, 'admin', '2017-03-10 18:29:37', NULL, NULL);
INSERT INTO `t_sys_sms` VALUES ('402881f3646a472b01646a4a5af00001', '催办：HR审批', '3', 'admin', NULL, 'admin，您好！\r\n请前待办任务办理事项！HR审批\r\n\r\n\r\n===========================\r\n此消息由系统发出', '2018-07-05 19:53:35', '2', NULL, NULL, NULL, 'admin', '2018-07-05 19:53:35', 'admin', '2018-07-07 13:45:24');
INSERT INTO `t_sys_sms` VALUES ('402881f3647da06c01647da43a940014', '催办：HR审批', '3', 'admin', NULL, 'admin，您好！\r\n请前待办任务办理事项！HR审批\r\n\r\n\r\n===========================\r\n此消息由系统发出', '2018-07-09 14:04:32', '2', NULL, NULL, NULL, 'admin', '2018-07-09 14:04:32', 'admin', '2018-07-09 18:51:30');
